/* bb_pdb.c -- Player statistics database

	M. Mallett	April 1990

The code in this file manages the player statistics database.

Statistics are assumed to be kept in external files representing
total stats as of the end of certain periods.  These database
routines keep track of those periodic statistics.  Routines herein
do not care what the period intervals are, merely that there are
a number of them.  Interface routines allow loading of the data
for specific periods, one at a time, and retrieving stats for
a particular range of intervals.

Included:

	pdb_init	Initialize database
	pdb_read	Read a stat file for a particular interval
	pdb_name	Get full name for player
	pdb_allstats	Get total stats for a player
	pdb_batstats	Get player's batting stats
	pdb_pitchstats	Get player's pitching stats.
	pdb_status Set status stuff for player

	era		Calc ERA from pitcher stats
	ratio		Calc ratio from pitcher stats
	avg		Calc average from batter stats
	

*/

#include <stdio.h>
#include <ctype.h>
#include <avl.h>
#include <string.h>

#ifdef __TURBOC__
#include <alloc.h>
#include <stdlib.h>
#else
#include <malloc.h>
#endif

#include "rotise.h"

#include "bb.h"
#include "bb_pdb.h"
#include "rdb.h"

/* Local definitions */

#define	RDBUFSIZE	200		/* Read buffer size */

    /* Structure for tree header */
typedef
  struct {
    AVLTREE	pd_tree;		/* AVL tree header */
    int		pd_firstint;		/* # of first interval */
    int		pd_lastint;		/* # of last interval */
  }    PDHDR;


    /* Structure for per-interval player data kept internally */
typedef
  struct {
    char	pd_team[4];		/* Team abbreviation */
	 char batter;			/* TRUE if batter struct following */
	union {
    PDB_BATTER	pd_batter;		/* Batter stats */
    PDB_PITCHER	pd_pitcher;		/* Pitcher stats */
    } pd_stats;
  }   INTSTAT;

    /* Structure of a player node in the AVL tree */
typedef
  struct {
    AVLNODE	pn_avl;			/* AVL header */
    char	pn_team[4];		/* First seen team name */
    INTSTAT	*pn_istatP;		/* Ptr to interval stats */
    char	*pn_lnameP;		/* Last name */
    char	*pn_fnameP;		/* First name */
    UWORD	pn_pos;			/* Eligible positions mask */
    char 	batter;			/* == TRUE if batter */
    PDATA	pdata;			/* Player's data */
  }    PNODE;


    /* Structure of field locator for raw data */
typedef
  struct {
    char	*t_nameP;		/* Field name in file header line */
    WORD	*t_vecP;		/* Ptr to vector field */
  }    FLDLOC;


    /* Parameter block for pdb_read continuation routine */
typedef
  struct {
    int		p_intN;			/* Interval */
    char	*p_fnameP;		/* Filename */
  }    READPB;


/* External data referenced */
extern int InputLine;

/* External routines used */



/* Local data publicly available */


/* Local routines and forward references */
static	int	statfixp PROTO( (int, int, char**, int, char *, int*) );
static	int	statint PROTO( (int, char **, int, char*, int*) );


    /* pdb_read() support routines */
static	int	pdb_batter PROTO( (char *, int, int, char **) );
static	int	pdb_btrhdr PROTO( (int, char**) );
static	int	pdb_pchhdr PROTO( (int, char**) );
static	int	pdb_pitcher PROTO( (char *, int, int, char**) );

    /* AVL support routines */
static	int	pd_cmpkey();
static	AVLNODE	*pd_mknode();
static	int	pd_rmnode();
static	int	pdw_each();

/* Private data */

static	PDHDR	Pdheader;		/* Data tree header */
static	int	(*Pdw_rtc)();		/* RTC for pdb_walk */
static	char	Readbuf[RDBUFSIZE];	/* Buffer for data reading */

    /* Structure containing token numbers for the batter stats */
static struct {
    WORD	bv_atbat;		/* # of atbats */
    WORD	bv_hits;		/* Hits */
    WORD	bv_1b;			/* Singles */
    WORD	bv_2b;			/* Doubles */
    WORD	bv_3b;			/* Triples */
    WORD	bv_hr;			/* Home Runs */
    WORD	bv_bb;			/* Walks */
    WORD	bv_hpb;			/* Hit by pitch */
    WORD	bv_rbi;			/* Runs batted in */
    WORD	bv_sb;			/* Stolen bases */
    WORD	bv_cs;			/* Caught stealing */
	}	Batvec;

    /* Structure containing token numbers for the batter stats */
static struct {
    WORD	pv_outs;		/* Outs (innings * 3) */
    WORD	pv_er;			/* Earned runs */
    WORD	pv_bb;			/* Walks */
    WORD	pv_hits;		/* Hits */
    WORD	pv_won;			/* Wins */
    WORD	pv_lost;		/* Losses */
    WORD	pv_saved;		/* Saves */
	}	Pchvec;


    /* Table of header names and Batvec elements */
static	FLDLOC	Bvttl[] = {
	{ "AB",		&Batvec.bv_atbat	},
	{ "H",		&Batvec.bv_hits		},
/*	{ "1B",		&Batvec.bv_1b		},    */
	{ "2B",		&Batvec.bv_2b		},
	{ "3B",		&Batvec.bv_3b		},
	{ "HR",		&Batvec.bv_hr		},
	{ "BB",		&Batvec.bv_bb		},
/*	{ "HPB",	&Batvec.bv_hpb		},     */
	{ "RBI",	&Batvec.bv_rbi		},
	{ "SB",		&Batvec.bv_sb		},
	{ "CS",		&Batvec.bv_cs		},
	{ NULL,		NULL			}

			};


    /* Table of header names and Pchvec elements */
static	FLDLOC	Pvttl[] = {
	{ "INN",	&Pchvec.pv_outs		},
	{ "ER",		&Pchvec.pv_er		},
	{ "BB",		&Pchvec.pv_bb		},
	{ "H",		&Pchvec.pv_hits		},
	{ "W",		&Pchvec.pv_won		},
	{ "L",		&Pchvec.pv_lost		},
	{ "SV",		&Pchvec.pv_saved	},
	{ NULL,		NULL			}

			};


    /* Table of header names and Posvec elements */
static struct {
	char *p_nameP;
	WORD  p_vec;
	WORD  p_mask;
	}		Posvttl[] = {

	{ "C",		0,	POS_C		},
	{ "1B",		0,	POS_1B		},
	{ "2B",		0,	POS_2B		},
	{ "3B",		0,	POS_3B		},
	{ "SS",		0,	POS_SS		},
	{ "OF",		0,	POS_OF		},
	{ "DH",		0,	POS_DH		},
	{ NULL,		0,		0		}

				};



    /* Table of team name abbreviations and full team names */
static char	*Teamtbl[] = {
	"BAL",	"Baltimore",	"Orioles",	"",
	"BOS",	"Boston",	"Red",	"Sox", "",
	"CAL",	"California",	"Angels",	"",
	"CHW",	"Chicago",	"White",	"Sox", "",
	"CLE",	"Cleveland",	"Indians",	"",
	"DET",	"Detroit",	"Tigers",	"",
	"KC",		"Kansas",	"City",		"Royals",	"",
	"MIL",	"Milwaukee",	"Brewers",	"",
	"MIN",	"Minnesota",	"Twins",	"",
	"NYY",	"New",		"York",		"Yankees",	"",
	"OAK",	"Oakland",	"Athletics",	"",
	"SEA",	"Seattle",	"Mariners",	"",
	"TEX",	"Texas",	"Rangers",	"",
	"TOR",	"Toronto",	"Blue",	"Jays", "",
	NULL
			};


/*

*//* pdb_init( firstint, lastint )

	Initialize the pdb module

Accepts :

	firstint		Number of first interval (typically 1)
	lastint			Number of last interval

Returns :

	< nothing >

*/

void
pdb_init( firstint, lastint )
	int		firstint;	/* First interval */
	int		lastint;	/* Last interval */
{
    /* Create the AVl tree */
    avlnew( &Pdheader.pd_tree, pd_cmpkey, pd_mknode, pd_rmnode );

    /* Fill in the header */
    Pdheader.pd_firstint = firstint;
    Pdheader.pd_lastint = lastint;
}
/*

*//* pdb_allstats( nameP, firstint, lastint, bstatP, pstatP )

	Retrieve all stats for a player

Accepts :

	nameP		Ptr to name block
	firstint	First interval
	lastint		Last interval
	bstatP		Ptr to PDB_BATTER block for batter stats
	pstatP		Ptr to PDB_PITCHER block for pitcher stats

Returns :

	<value >	TRUE if OK
			FALSE if problem
	*bstatP		Filled in
	*pstatP		Filled in

*/

int
pdb_allstats( nameP, firstint, lastint, bstatP, pstatP )
	PDB_PNAME	*nameP;		/* Player name block */
	int		firstint;	/* First interval */
	int		lastint;	/* Last interval */
	PDBT_BATTER	*bstatP;	/* Ptr to batter stat block */
	PDBT_PITCHER	*pstatP;	/* Ptr to pitcher stat block */
{
    if ( !pdb_batstats( nameP, firstint, lastint, bstatP ) )
	return( FALSE );
    return( pdb_pitchstats( nameP, firstint, lastint, pstatP ) );
}
/*

*//* pdb_batstats( nameP, firstint, lastint, bstatP )

	Retrieve batting stats for a player

Accepts :

	nameP		Ptr to name block
	firstint	First interval
	lastint		Last interval
	bstatP		Ptr to PDB_BATTER block for batter stats

Returns :

	<value >	TRUE if OK
			FALSE if problem
	*bstatP		Filled in

*/
int
pdb_batstats( nameP, firstint, lastint, bstatP )
	PDB_PNAME	*nameP;		/* Ptr to nameblock */
	int		firstint;	/* First interval */
	int		lastint;	/* Last interval */
	PDBT_BATTER	*bstatP;	/* Ptr to batter stat block */
{
	PNODE		*nodeP;		/* Ptr to player node */
	PDB_BATTER	*pbstatP;	/* Ptr to player stat line */
	char		msgbuf[80];	/* General buffer */

    /* Make sure the interval range is OK */
    if ( ( firstint < ( Pdheader.pd_firstint -1 ) ) ||
         ( lastint > Pdheader.pd_lastint ) ) {
	rdb_error( "pdb_batstat(): invalid interval range.", "" );
	return( FALSE );
    }

	if ( lastint < Pdheader.pd_firstint )
		return FALSE;

	 nameP->batter = TRUE;
    /* Lookup the player data */
    nodeP = (PNODE *)avlfind( &Pdheader.pd_tree, nameP );
    if ( nodeP == NULL ) {
	sprintf( &msgbuf[0], " %s, %s (%s)",
		&nameP->pn_lname[0], &nameP->pn_fname[0], &nameP->pn_team[0] );
	rdb_error( "No such batter: ", &msgbuf[0] );
	return( FALSE );
    }

    /* Copy player stats as of last interval */
    pbstatP = &nodeP->pn_istatP[ lastint - Pdheader.pd_firstint ].pd_stats.pd_batter;

	bstatP->bs_atbat = pbstatP->bs_atbat;
	bstatP->bs_hits = pbstatP->bs_hits;
	bstatP->bs_1b = pbstatP->bs_1b;
	bstatP->bs_2b = pbstatP->bs_2b;
	bstatP->bs_3b = pbstatP->bs_3b;
	bstatP->bs_hr = pbstatP->bs_hr;
	bstatP->bs_bb = pbstatP->bs_bb;
	bstatP->bs_hpb = pbstatP->bs_hpb;
	bstatP->bs_rbi = pbstatP->bs_rbi;
	bstatP->bs_sb = pbstatP->bs_sb;
	bstatP->bs_cs = pbstatP->bs_cs;

    /* Subtract out base interval if it's within first-last range */
    if ( firstint >= Pdheader.pd_firstint ) {
	pbstatP = &nodeP->pn_istatP[ firstint - Pdheader.pd_firstint ].pd_stats.pd_batter;

	bstatP->bs_atbat -= pbstatP->bs_atbat;
	bstatP->bs_hits -= pbstatP->bs_hits;
	bstatP->bs_1b -= pbstatP->bs_1b;
	bstatP->bs_2b -= pbstatP->bs_2b;
	bstatP->bs_3b -= pbstatP->bs_3b;
	bstatP->bs_hr -= pbstatP->bs_hr;
	bstatP->bs_bb -= pbstatP->bs_bb;
	bstatP->bs_hpb -= pbstatP->bs_hpb;
	bstatP->bs_rbi -= pbstatP->bs_rbi;
	bstatP->bs_sb -= pbstatP->bs_sb;
	bstatP->bs_cs -= pbstatP->bs_cs;
    }

    /* We don't really have 1b -- fix it up here. */
    bstatP->bs_1b = bstatP->bs_hits - (bstatP->bs_hr + bstatP->bs_2b +
					bstatP->bs_3b);

    return( TRUE );
}
/*

*//* pdb_pitchstats( nameP, firstint, lastint, pstatP )

	Retrieve pitching stats for a player

Accepts :

	nameP		Ptr to name block
	firstint	First interval
	lastint		Last interval
	pstatP		Ptr to PDB_PITCHER block for pitcher stats

Returns :

	<value >	TRUE if OK
			FALSE if problem
	*pstatP		Filled in

*/

int
pdb_pitchstats( nameP, firstint, lastint, pstatP )
	PDB_PNAME	*nameP;		/* Ptr to nameblock */
	int		firstint;	/* First interval */
	int		lastint;	/* Last interval */
	PDBT_PITCHER	*pstatP;	/* Ptr to pitcher stat block */
{
	PNODE		*nodeP;		/* Ptr to player node */
	PDB_PITCHER	*ppstatP;	/* Ptr to player stat line */
	char		msgbuf[80];	/* For formatting strings */

    /* Make sure the interval range is OK */
    if ( ( firstint < ( Pdheader.pd_firstint -1 ) ) ||
         ( lastint > Pdheader.pd_lastint ) ) {
	rdb_error( "pdb_pitchstat(): invalid interval range.", "" );
	return( FALSE );
    }

	if ( lastint < Pdheader.pd_firstint )
		return FALSE;

    /* Lookup the player data */
	 nameP->batter = FALSE;
    nodeP = (PNODE *)avlfind( &Pdheader.pd_tree, nameP );
    if ( nodeP == NULL ) {
	sprintf( &msgbuf[0], "%s, %s (%s)",
		&nameP->pn_lname[0], &nameP->pn_fname[0], &nameP->pn_team[0] );
	rdb_error( "No such pitcher: ", &msgbuf[0] );
	return( FALSE );
    }

    /* Copy player stats as of last interval */
    ppstatP = &nodeP->pn_istatP[ lastint - Pdheader.pd_firstint ].pd_stats.pd_pitcher;
	pstatP->ps_outs = ppstatP->ps_outs;
	pstatP->ps_er = ppstatP->ps_er;
	pstatP->ps_bb = ppstatP->ps_bb;
	pstatP->ps_hits = ppstatP->ps_hits;
	pstatP->ps_won = ppstatP->ps_won;
	pstatP->ps_lost = ppstatP->ps_lost;
	pstatP->ps_saved = ppstatP->ps_saved;

    /* Subtract out base interval if it's within first-last range */
    if ( firstint >= Pdheader.pd_firstint ) {
	ppstatP = &nodeP->pn_istatP[ firstint - Pdheader.pd_firstint ].pd_stats.pd_pitcher;

	pstatP->ps_outs -= ppstatP->ps_outs;
	pstatP->ps_er -= ppstatP->ps_er;
	pstatP->ps_bb -= ppstatP->ps_bb;
	pstatP->ps_hits -= ppstatP->ps_hits;
	pstatP->ps_won -= ppstatP->ps_won;
	pstatP->ps_lost -= ppstatP->ps_lost;
	pstatP->ps_saved -= ppstatP->ps_saved;
    }

    return( TRUE );
}
/*
/*

*//* pdb_create_player( nameP )

	Create a player in the PDB database.

Accepts :

	nameP		Ptr to name block for new player.

Returns :

	<value >	TRUE if OK
			FALSE if problem
*/

int
pdb_create_player ARGLIST ( ( nameP ) )
     FARG( PDB_PNAME	*nameP )	/* Ptr to nameblock */
{
	PNODE		*nodeP;		/* Ptr to player node */

    /* Attempt to make node whether it exists or not */
    avlinsert( &Pdheader.pd_tree, nameP, (void *)NULL );
    
    /* Let's see if it's there now. */
    if ( (nodeP = (PNODE *)avlfind( &Pdheader.pd_tree, nameP ) ) == NULL )
	return ( FALSE );

    /* Make sure pitcher is marked in position mask. */
    if ( !nameP->batter )
	nodeP->pn_pos = POS_P;

    return ( TRUE );
}
/*

*//* pdb_name( nameP, intN, fnameP )

	Retrieve a full name for a player

Accepts :

	nameP		Ptr to name block to look up
	intN		Interval to get name(team) for (or 0 for base)
	fnameP		Ptr to name block to receive the info

Returns :

	<value >	TRUE if OK
			FALSE if problem
	*fnameP		Filled in
*/

int
pdb_name( nameP, intN, fnameP )
	PDB_PNAME	*nameP;		/* Ptr to nameblock */
	int		intN;		/* Interval */
	PDB_PNAME	*fnameP;	/* Ptr to fullname block */
{
	PNODE		*nodeP;		/* Ptr to player node */

    /* Make sure the interval is OK */
    if ( ( intN < 0 ) || ( intN > Pdheader.pd_lastint ) ) {
	rdb_error( "pdb_name(): invalid interval.", "" );
	return( FALSE );
    }

    /* Lookup the player data */
    nodeP = (PNODE *)avlfind( &Pdheader.pd_tree, nameP );
    if ( nodeP == NULL )
	return( FALSE );

    /* Copy the name */
    strcpy( &fnameP->pn_lname[0], nodeP->pn_lnameP );
    strcpy( &fnameP->pn_fname[0], nodeP->pn_fnameP );
    if ( intN == 0 )
	strcpy( &fnameP->pn_team[0], &nodeP->pn_team[0] );
    else
	strcpy( &fnameP->pn_team[0],
		&nodeP->pn_istatP[ intN - Pdheader.pd_firstint ].pd_team[0] );

    return( TRUE );
}
/*

*//* pdb_getpos( nameP, posP )

	Retrieve position eligibility mask for a player

Accepts :

	nameP		Ptr to name block to look up
	posP		Addr of return variable to receive the mask

Returns :

	<value >	TRUE if OK
			FALSE if problem
	*posP		Filled in if valid player
*/

int
pdb_getpos ARGLIST( ( nameP, posP ) )
    NFARG( PDB_PNAME *nameP )
     FARG( UWORD *posP )
{
	PNODE		*nodeP;		/* Ptr to player node */

    /* Lookup the player data */
    nodeP = (PNODE *)avlfind( &Pdheader.pd_tree, nameP );
    if ( nodeP == NULL )
	return( FALSE );

    /* Copy the position mask */
    *posP = nodeP->pn_pos;

    return ( TRUE );
}
/*

*//* pdb_setpos( nameP, pos )

	Set position information for a player

Accepts :

	nameP		Ptr to name block to set position in
	pos		Mask of eligible positions

Returns :

	<value >	TRUE if OK
			FALSE if problem
*/

int
pdb_setpos ARGLIST( ( nameP, pos ) )
    NFARG( PDB_PNAME *nameP )
     FARG( UWORD pos )
{
	PNODE		*nodeP;		/* Ptr to player node */

    /* Lookup the player data */
    nodeP = (PNODE *)avlfind( &Pdheader.pd_tree, nameP );
    if ( nodeP == NULL )
	return( FALSE );

    /* Set a new position */
    nodeP->pn_pos = pos;

    return ( TRUE );
}
/*

*//* pdb_walk( rtc )

	Walk the player tree

Accepts :

	rtc		Routine to call for each player.  (See Note)

Returns :

	< nothing >

Notes :

	The routine will be passed a PDB_PNAME struct with the player info.
	Routine should return TRUE to continue, FALSE to stop.

	Routine MUST NOT alter the tree without aborting.

*/


void
pdb_walk ARGLIST( (rtc) )
     FARG( int (*rtc) PROTO( ( PDB_PNAME *) ) )
{
    Pdw_rtc = rtc;			/* Retain rtc */

    /* Do avlwalk via the local pdw_each() routine */
    avlwalk( &Pdheader.pd_tree, pdw_each );
}

    /* The "each" routine for avlwalk for pdb_walk */
static int
pdw_each ARGLIST( ( treeP, nodeP, depth ) )
    NFARG( AVLTREE *treeP )
    NFARG( PNODE *nodeP )
     FARG( int depth )
{
	int		status;
	PDB_PNAME	nameblk;

    if ( nodeP == NULL )
    	status = (*Pdw_rtc)( (PDB_PNAME *)NULL );
    else {
	/* Construct name block */
	strcpy( &nameblk.pn_lname[0], nodeP->pn_lnameP );
	strcpy( &nameblk.pn_fname[0], nodeP->pn_fnameP );
	strcpy( &nameblk.pn_team[0], "" );
	nameblk.batter = nodeP->batter;

	/* Pass it to the walk routine */
	status = (*Pdw_rtc)( &nameblk );
    }

    return ( status ? 0 : 1 );
}
/*

*//* pdb_read( intN, fnameP )

	Read an interval's data

Accepts :

	intN		Interval number
	fnameP		Name of the stats file

Returns :

	< value >	TRUE if success
			FALSE if failure

Notes :

	This routine is specifically designed to work with a particular
	format file.  Some flexibility in column order is built in, but
	certain major sections must appear in correct order and form.

*/

int
pdb_read( intN, fnameP )
DREG1	int		intN;		/* Interval number */
	char		*fnameP;	/* Name of stats file */
{
	int		successF;	/* If successful */
	int		i;		/* Scratch */
	int		tokC;		/* Number of tokens */
AREG1	char		*teamP;		/* Ptr to team name */
	char		**teamPP;	/* Team name ptr ptr */
	FILE		*fP;		/* Ptr to file */
	char		*tokV[50];	/* Token vector */
	char		tokbuf[RDBUFSIZE];	/* Buffer for tokens */

DREG2	enum {
		SEC_BLURB,
		SEC_TEAM,
		SEC_BATHDR,
		SEC_BATTER,
		SEC_PCHHDR,
		SEC_PITCHER
	     }		nextsection;

	char		msgbuf[80];	/* For message... */


    /* Init line number to 0 for error reporting */
    InputLine = 0;

    /* Check out interval number */
    if ( ( intN < Pdheader.pd_firstint ) ||
         ( intN > Pdheader.pd_lastint ) ) {
	sprintf( &msgbuf[0], "pdb_read: interval %d out of range (%d thru %d).",
			intN, Pdheader.pd_firstint, Pdheader.pd_lastint );
	rdb_error( &msgbuf[0], "" );
	return( FALSE );
    }

    /* Access the file */
    if ( ( fP = fopen( fnameP, "r" ) ) == NULL ) {
	rdb_error( "pdb_read: Can't access file: ", fnameP );
	return( FALSE );
    }

    /* Loop getting team stats */
    successF = TRUE;
    nextsection = SEC_BLURB;
    for( InputLine = 1;getline( fP, &Readbuf[0], RDBUFSIZE ) >= 0; ++InputLine ) {
	tokC = tkline( &Readbuf[0], &tokbuf[0], &tokV[0], 50, "", " \t," );
	/* Skip blank lines */
	if ( tokC == 0 ) {
	    if ( nextsection == SEC_BATTER )
		nextsection = SEC_PCHHDR;
	    else if ( nextsection == SEC_PITCHER )
		nextsection = SEC_BLURB;
	    continue;
	}

	/* Process according to what is expected next */
	switch( nextsection ) {
	    case SEC_BLURB:
		/* Throw out "From the ..." notice */
		if ( ( strcmp( tokV[0], "From" ) == 0 ) &&
		     ( strcmp( tokV[1], "the" ) == 0 ) &&
	 	     ( strcmp( tokV[2], "Elias" ) == 0 ) )
		    nextsection = SEC_TEAM;
		else 
		    rdb_error( "pdb_read(): expected blurb line.", "" );
		break;

	    case SEC_TEAM:
		/* Find the team name */
		teamPP = &Teamtbl[0];
		while( *teamPP != NULL ) {
		    for( i = 0; i < tokC; ++i ) {
			teamP = teamPP[i+1];
			if ( ( *teamP == NUL ) ||
			     ( stricmp( teamP, tokV[i] ) != 0 ) )
			    break;
		    }
		    if ( ( i == tokC ) && ( *teamPP[i+1] == NUL ) )
			break;

		    while ( **teamPP != NUL )
			++teamPP;
		    ++teamPP;
		}

		teamP = *teamPP;
		if ( teamP == NULL )
		    rdb_error( "pdb_read(): unrecognized team.", "" );
		nextsection = SEC_BATHDR;
		break;


	    case SEC_BATHDR:
		/* Must be batter header line */
		if ( stricmp( tokV[0], "batters" ) != 0 )
		    rdb_error( "pdb_read(): expected BATTERS line.", "" );
		else {
		    successF = pdb_btrhdr( tokC, &tokV[0] ) && successF;
		    nextsection = SEC_BATTER;
		}
		break;
					
	    case SEC_BATTER:
		successF = pdb_batter( teamP, intN, tokC, &tokV[0] );
		break;

	    case SEC_PCHHDR:
		/* Must be pitcher header line */
		if ( stricmp( tokV[0], "pitchers" ) != 0 )
		    rdb_error( "pdb_read(): expected PITCHERS line.", "" );
		else {
		    successF = pdb_pchhdr( tokC, &tokV[0] ) && successF;
		    nextsection = SEC_PITCHER;
		}
		break;
					
	    case SEC_PITCHER:
		successF = pdb_pitcher( teamP, intN, tokC, &tokV[0] );
		break;
	 }
    }

    fclose( fP );

    return ( successF );
}
/*

*//* pdb_readpos( fnameP, mingames )

	Read position information

Accepts :

	fnameP		Name of the positions file
	mingames	Minimum # of games to qualify for position

Returns :

	< value >	TRUE if success
			FALSE if failure

Notes :

	This routine is specifically designed to work with a particular
	format file.  Some flexibility in column order is built in, but
	certain major sections must appear in correct order and form.

	Team information in position file is ignore: the important data
	is the positions.

*/

int
pdb_readpos ARGLIST( (fnameP, mingames) )
    NFARG( char		*fnameP )	/* Position file name */
     FARG( int		mingames )	/* Qualifying threshold */
{
	char		headerF;	/* If looking for header */
	int		successF;	/* If successful */
	int		i;		/* Scratch */
	int		tokC;		/* Number of tokens */
AREG1	char		*teamP;		/* Ptr to team name */
	char		**teamPP;	/* Team name ptr ptr */
	FILE		*fP;		/* Ptr to file */
	char		*tokV[50];	/* Token vector */
	char		tokbuf[RDBUFSIZE];	/* Buffer for tokens */

    /* Init line number to 0 for error reporting */
    InputLine = 0;

    /* Access the file */
    if ( ( fP = fopen( fnameP, "r" ) ) == NULL ) {
	rdb_error( "pdb_readpos: Can't access file: ", fnameP );
	return( FALSE );
    }

    /* Loop getting position information */
    successF = TRUE;
    headerF = TRUE;			/* Looking for header */
    for( InputLine = 1;getline( fP, &Readbuf[0], RDBUFSIZE ) >= 0; ++InputLine ) {
	tokC = tkline( &Readbuf[0], &tokbuf[0], &tokV[0], 50, "", " \t," );

	/* This line blank? */
	if ( tokC == 0 ) {
	    headerF = TRUE;		/* Expect header next */
	    continue;
	}

	/* Looking for header? */
	if ( headerF ) {
	    if ( stricmp( tokV[0], "PLAYER" ) != 0 )
		continue;		/* Didn't find it yet */

	    /* Decode the column information */
	    pdb_poshdr( tokC, tokV );
	    headerF = FALSE;		/* Done with header. */
	}
	else {
	    /* Not a header line: process position info. */
	    pdb_pos( tokC, tokV, mingames );
	}
    }

    fclose( fP );

    return ( successF );
}

/*
	BOOL pdb_status( PDB_PNAME *nameP, char *st_str, char *st_date, BOOL free, int week )
	 Set the status stuff for the player in the AVL tree.  No error checking
	is done here, so the strings had better not be too big.

 ACCEPTS:
	PDB_PNAME *nameP - ptr to name block to look up and set
	char *st_str - status string to set
	char *st_date - date string to set
	BOOL free - TRUE if free agent
	int week - effective start week for status

 RETURNS:
	BOOL status - FALSE if couldn't find player, TRUE otherwise

*/
BOOL pdb_status ARGLIST( (nameP, st_str, st_date, free, week) )
	NFARG( PDB_PNAME *nameP )
	NFARG( char *st_str )
	NFARG( char *st_date )
	NFARG( BOOL free )
	FARG( int week )
{
	PNODE		*nodeP;		/* Ptr to player's node */

	/* Lookup the player data */
	nodeP = (PNODE *)avlfind( &Pdheader.pd_tree, nameP );

	if ( nodeP == NULL )
	{ /* couldn't find the player */
		return FALSE;
	}

	strcpy( nodeP->pdata.status, st_str );
	strcpy( nodeP->pdata.date, st_date );
	nodeP->pdata.week = week;
	nodeP->pdata.zorch = free;

	return TRUE;
}


/*
	PDATA *pdb_pdata( char *lname, char *fname, BOOL batter )
	 Return a pointer to the PDATA structure for the passed player key.
	If player doesn't exist, return NULL.

 ACCEPTS:
	char *lname - last name of player key
	char *fname - first name of player key
	BOOL batter - TRUE if looking for batter, FALSE for pitcher

 RETURNS:
	PDATA * - NULL if not found, otherwise ptr to data in AVL tree

*/
PDATA *pdb_pdata ARGLIST(  (lname, fname, batter) )
	NFARG( 	char *lname )
	NFARG( 	char *fname )
	FARG(		BOOL batter )
{
	PNODE		*nodeP;		/* Ptr to player's node */
	PDB_PNAME pname;		/* Fully qualified name key */

	strcpy( pname.pn_lname, lname );
	strcpy( pname.pn_fname, fname );

	pname.batter = batter;

	/* Lookup the player data */
	nodeP = (PNODE *)avlfind( &Pdheader.pd_tree, &pname );

	if ( nodeP == NULL )
	{ /* couldn't find the player */
		return NULL;
	}

	return &nodeP->pdata;
}

/*

*//* Calculation routines 

	Some useful routines for calculating derived values.

*/

#ifdef __ANSI__
float era( PDBT_PITCHER *pchP )
#else
float era( pchP )
	PDBT_PITCHER		*pchP;
#endif
{
    if ( pchP->ps_outs == 0 )
	return( pchP->ps_er == 0 ? 0.0 : 999.0 );

    return( ((float)pchP->ps_er * 27) / pchP->ps_outs );
}



#ifdef __ANSI__
float ratio( PDBT_PITCHER *pchP )
#else
float ratio( pchP )
	PDBT_PITCHER		*pchP;
#endif
{
	int		hnw;		/* Hits and walks */

    hnw = pchP->ps_hits + pchP->ps_bb;

    if ( pchP->ps_outs == 0 )
	return( hnw == 0 ? 0.0 : 999.0 );

    return( ((float)hnw * 3) / pchP->ps_outs );
}



#ifdef __ANSI__
float avg( PDBT_BATTER *batP )
#else
float avg( batP )
	PDBT_BATTER		*batP;
#endif
{
    if ( batP->bs_atbat == 0 )
	return( 0.0 );
    return( ((float)batP->bs_hits) / batP->bs_atbat );
}
/*

*//******************************************************************
*                                                                   *
*               Local Routines begin Here (here)                    *
*                                                                   *
*********************************************************************/

/*

*//* pdb_btrhdr( tokC, tokV )

	Process a batter header line

Accepts :

	tokC		Number of tokens
	tokV		Token vector

Returns :

	<value>		TRUE if success
			FALSE if any bad errors were reported

*/

static int
pdb_btrhdr( tokC, tokV )
	int		tokC;		/* Number of tokens */
AREG1	char		**tokV;		/* Token vector */
{
	int		status;
DREG1	int		vX;		/* Index to vector table */
DREG2	int		tX;		/* Index to token list */

    memset( &Batvec, 0, sizeof( Batvec ) );

    /* Find the fields in the vector list */
    status = TRUE;
    for( vX = 0; Bvttl[vX].t_nameP != NULL; ++vX ) {
	for( tX = 1; tX < tokC; ++tX )
	    if ( stricmp( tokV[tX], Bvttl[vX].t_nameP ) == 0 )
		break;

	if ( tX < tokC )
	    /* Found the title; store field index (plus one, because
	        the player name takes two tokens) */
	    *Bvttl[vX].t_vecP = tX +1;
	else {
	    rdb_error( "pdb_btrhdr(): missing field ", 
				Bvttl[vX].t_nameP );
	    status = FALSE;
	}
    }
    
    return( status );
}
/*

*//* pdb_batter( teamP, intN, tokC, tokV )

	Process a batter's stats

Accepts :

	teamP		Ptr to team abbreviation
	intN		Interval  number
	tokC		Number of tokens
	tokV		Token vector

Returns :

	< value >	TRUE if no problems
			FALSE if bad error reported

*/

static int
pdb_batter( teamP, intN, tokC, tokV )
	char		*teamP;		/* Team abbreviation */
	int		intN;		/* Interval number */
	int		tokC;		/* Token count */
AREG1	char		**tokV;		/* Token vector */
{
	int		status;
	int		i;
	PNODE		*nodeP;		/* Ptr to player's node */
	PDB_PNAME	nameblk;	/* Block for avl insert/lookup */
AREG2	INTSTAT		*statP;		/* Ptr to particular stat line */
	char		msgbuf[80];

/* printf( "batter: %s, %s\n", tokV[0], tokV[1] ); */

    /* Check missing first initial */
    if ( *tokV[1] == '.' || isdigit( *tokV[1] ) ) {
	sprintf( &msgbuf[0], "Missing first name for \"%s\" -- using \".\"",
				tokV[0] );
	rdb_error( &msgbuf[0], "" );
	for( i = tokC++; i > 1; --i )
	    tokV[i] = tokV[i-1];
	tokV[1] = ".";
    }

    /* Construct name block for avl lookup */
    memset( &nameblk.pn_team[0], 0, 4 );
    strncpy( &nameblk.pn_team[0], teamP, 3 );
    strcpy( &nameblk.pn_lname[0], tokV[0] );
    strcpy( &nameblk.pn_fname[0], tokV[1] );

    /* Try to find this guy */
	 nameblk.batter = TRUE;
    nodeP = (PNODE *)avlfind( &Pdheader.pd_tree, &nameblk );
    if ( nodeP == NULL ) {
	/* Not there... try to insert it */
	avlinsert( &Pdheader.pd_tree, &nameblk, (void *)NULL );
	nodeP = (PNODE *)avlfind( &Pdheader.pd_tree, &nameblk );
	nodeP->pdata.zorch = PLAYER_FREE;
    }
	else if ( nodeP->pdata.zorch == PLAYER_UNAVAILABLE )
	{ /* mark as available */
	 nodeP->pdata.zorch = PLAYER_FREE;
	}
    if ( nodeP == NULL ) {
    	sprintf( &msgbuf[0], "pdb_batter(): avl failure for %s, %c",
		tokV[0], *tokV[1] );
	rdb_error( &msgbuf[0], "" );
	return( FALSE );
    }

    /* Fill in the stat line for this interval */
    status = TRUE;
    statP = &nodeP->pn_istatP[ intN - Pdheader.pd_firstint ];

    strcpy( &statP->pd_team[0], &nameblk.pn_team[0] );

	 statP->batter = TRUE;					/* Show we have a batter */
    statP->pd_stats.pd_batter.bs_atbat = statint( tokC, tokV, Batvec.bv_atbat, "at-bats", &status );
    statP->pd_stats.pd_batter.bs_hits = statint( tokC, tokV, Batvec.bv_hits, "hits", &status );
    statP->pd_stats.pd_batter.bs_1b = statint( tokC, tokV, Batvec.bv_1b, "1b", &status );
    statP->pd_stats.pd_batter.bs_2b = statint( tokC, tokV, Batvec.bv_2b, "2b", &status );
    statP->pd_stats.pd_batter.bs_3b = statint( tokC, tokV, Batvec.bv_3b, "3b", &status );
    statP->pd_stats.pd_batter.bs_hr = statint( tokC, tokV, Batvec.bv_hr, "hr", &status );
    statP->pd_stats.pd_batter.bs_bb = statint( tokC, tokV, Batvec.bv_bb, "bb", &status );
    statP->pd_stats.pd_batter.bs_hpb = statint( tokC, tokV, Batvec.bv_hpb, "hpb", &status );
    statP->pd_stats.pd_batter.bs_rbi = statint( tokC, tokV, Batvec.bv_rbi, "rbi", &status );
    statP->pd_stats.pd_batter.bs_sb = statint( tokC, tokV, Batvec.bv_sb, "sb", &status );
    statP->pd_stats.pd_batter.bs_cs = statint( tokC, tokV, Batvec.bv_cs, "cs", &status );

    return( status );
}
/*

*//* pdb_pchhdr( tokC, tokV )

	Process a pitcher header line

Accepts :

	tokC		Number of tokens
	tokV		Token vector

Returns :

	<value>		TRUE if success
			FALSE if any bad errors were reported

*/

static int
pdb_pchhdr( tokC, tokV )
	int		tokC;		/* Number of tokens */
	char		**tokV;		/* Token vector */
{
	int		status;
DREG1	int		vX;		/* Index to vector table */
DREG2	int		tX;		/* Index to token list */

    memset( &Pchvec, 0, sizeof( Pchvec ) );

    /* Find the fields in the vector list */
    status = TRUE;
    for( vX = 0; Pvttl[vX].t_nameP != NULL; ++vX ) {
	for( tX = 1; tX < tokC; ++tX )
	    if ( stricmp( tokV[tX], Pvttl[vX].t_nameP ) == 0 )
		break;

	if ( tX < tokC )
	    /* Found the title; store field index (plus one, because
	        the player name takes two tokens) */
	    *Pvttl[vX].t_vecP = tX +1;
	else {
	    rdb_error( "pdb_pchhdr(): missing field: ", Pvttl[vX].t_nameP );
	    status = FALSE;
	}
    }
    return( status );
}
/*

*//* pdb_pitcher( teamP, intN, tokC, tokV )

	Process a pitcher's stats

Accepts :

	teamP		Ptr to team abbreviation
	intN		Interval  number
	tokC		Number of tokens
	tokV		Token vector

Returns :

	< value >	TRUE if no problems
			FALSE if bad error reported

*/

static int
pdb_pitcher( teamP, intN, tokC, tokV )
	char		*teamP;		/* Team name abbrev. */
	int		intN;		/* Interval number */
	int		tokC;		/* Token count */
	char		**tokV;		/* Token vector */
{
	int		status;
	int		i;
	PNODE		*nodeP;		/* Ptr to player's node */
	PDB_PNAME	nameblk;	/* Block for avl insert/lookup */
AREG2	INTSTAT		*statP;		/* Ptr to particular stat line */
	char		msgbuf[80];

/* printf( "pitcher: %s, %s\n", tokV[0], tokV[1] ); */

    /* Check missing first initial */
    if ( *tokV[1] == '.' || isdigit( *tokV[1] ) ) {
	sprintf( &msgbuf[0], "Missing first name for \"%s\" -- using \".\"",
				tokV[0] );
	rdb_error( &msgbuf[0], "" );
	for( i = tokC++; i > 1; --i )
	    tokV[i] = tokV[i-1];
	tokV[1] = ".";
    }

    /* Construct name block for avl lookup */
    memset( &nameblk.pn_team[0], 0, 4 );
    strncpy( &nameblk.pn_team[0], teamP, 3 );
    strcpy( &nameblk.pn_lname[0], tokV[0] );
    strcpy( &nameblk.pn_fname[0], tokV[1] );
	 nameblk.batter = FALSE;

    /* Try to find this guy */
    nodeP = (PNODE *)avlfind( &Pdheader.pd_tree, &nameblk );
    if ( nodeP == NULL ) {
	/* Not there... try to insert it */
	avlinsert( &Pdheader.pd_tree, &nameblk, (void *)NULL );
	nodeP = (PNODE *)avlfind( &Pdheader.pd_tree, &nameblk );
	nodeP->pdata.zorch = PLAYER_FREE;
    }
    if ( nodeP == NULL ) {
    	sprintf( &msgbuf[0], "pdb_pitcher(): avl failure for %s, %c",
		tokV[0], *tokV[1] );
	rdb_error( &msgbuf[0], "" );
	return( FALSE );
    }

    /* We know one position, so mark it. */
    nodeP->pn_pos = POS_P;

    /* Fill in the stat line for this interval */
    status = TRUE;
    statP = &nodeP->pn_istatP[ intN - Pdheader.pd_firstint ];

    strcpy( &statP->pd_team[0], &nameblk.pn_team[0] );

    statP->batter = FALSE;			/* Pitcher here */
    statP->pd_stats.pd_pitcher.ps_outs = statfixp( 3, tokC, tokV, Pchvec.pv_outs, "outs", &status );
    statP->pd_stats.pd_pitcher.ps_er = statint( tokC, tokV, Pchvec.pv_er, "er", &status );
    statP->pd_stats.pd_pitcher.ps_bb = statint( tokC, tokV, Pchvec.pv_bb, "bb", &status );
    statP->pd_stats.pd_pitcher.ps_hits = statint( tokC, tokV, Pchvec.pv_hits, "hits", &status );
    statP->pd_stats.pd_pitcher.ps_won = statint( tokC, tokV, Pchvec.pv_won, "won", &status );
    statP->pd_stats.pd_pitcher.ps_lost = statint( tokC, tokV, Pchvec.pv_lost, "lost", &status );
    statP->pd_stats.pd_pitcher.ps_saved = statint( tokC, tokV, Pchvec.pv_saved, "saved", &status );

    return( status );
}
/*

*//* pdb_poshdr( tokC, tokV )

	Process a position file header line

Accepts :

	tokC		Number of tokens
	tokV		Token vector

Returns :

	<value>		TRUE if success
			FALSE if any bad errors were reported

*/

static int
pdb_poshdr( tokC, tokV )
	int		tokC;		/* Number of tokens */
AREG1	char		**tokV;		/* Token vector */
{
	int		status;
DREG1	int		vX;		/* Index to vector table */
DREG2	int		tX;		/* Index to token list */

    /* Find the fields in the vector list */
    status = TRUE;
    for( vX = 0; Posvttl[vX].p_nameP != NULL; ++vX ) {
	Posvttl[vX].p_vec = 0;
	for( tX = 1; tX < tokC; ++tX )
	    if ( stricmp( tokV[tX], Posvttl[vX].p_nameP ) == 0 )
		break;

	if ( tX < tokC )
	    /* Found the title; store field index (plus one, because
	        the player name takes two tokens) */
	    Posvttl[vX].p_vec = tX +1;
	else {
	    rdb_error( "pdb_poshdr(): missing field ", 
				Posvttl[vX].p_nameP );
	    status = FALSE;
	}
    }
    
    return( status );
}
/*

*//* pdb_pos( tokC, tokV, mingames )

	Process a player's positional information

Accepts :

	tokC		Number of tokens
	tokV		Token vector
	mingames	Min # games required at any position

Returns :

	< value >	TRUE if no problems
			FALSE if bad error reported

*/

static int
pdb_pos ARGLIST( ( tokC, tokV, mingames ) )
    NFARG( int		tokC )		/* Token count */
    NFARG( char		**tokV )	/* Tokens */
     FARG( int		mingames )	/* Game threshold */
{
	int		vX;		/* Vector table index */
	int		maxpos;		/* Maximum positions played */
	int		gp;		/* Games played at a position */
	UWORD		posmask;	/* Position mask */
	PNODE		*nodeP;		/* Ptr to player's node */
	PDB_PNAME	nameblk;	/* Block for avl insert/lookup */
	char		msgbuf[80];

    /* Check the max # played at any position */
    for( maxpos = 0, vX = 0; Posvttl[vX].p_nameP != NULL; ++vX ) {
	gp = atoi( tokV[ Posvttl[vX].p_vec ] );
	if ( gp > maxpos )
	    maxpos = gp;
    }

    /* If no games at any position, just ignore. */
    if ( maxpos == 0 )
	return ( TRUE );

    /* If max is less than the threshold, lower the threshold */
    if ( maxpos < mingames )
	mingames = maxpos;

    /* Build a mask of positions played above threshold */
    for( posmask = 0, vX = 0; Posvttl[vX].p_nameP != NULL; ++vX ) {
	gp = atoi( tokV[ Posvttl[vX].p_vec ] );
	if ( gp >= mingames )
	    posmask |= Posvttl[vX].p_mask;
    }

    /* Create or find player */
    strcpy( &nameblk.pn_lname[0], tokV[0] );
    strcpy( &nameblk.pn_fname[0], tokV[1] );
    strcpy( &nameblk.pn_team[0], "" );
    nameblk.batter = TRUE;
    avlinsert( &Pdheader.pd_tree, &nameblk, (void *)NULL );
    if ( (nodeP = (PNODE *)avlfind( &Pdheader.pd_tree, &nameblk ) ) == NULL ) {
	sprintf( &msgbuf[0], "%s, %s", tokV[0], tokV[1] );
	rdb_error( "pdb_pos: Can't make player ", &msgbuf[0] );
	return ( FALSE );
    }

    /* Update position mask */
    nodeP->pn_pos |= posmask;

    return ( TRUE );
}
/*

*//* statint( tokC, tokV, vX, fnameP, statusP )

	Obtain an integer statistic from raw data

Accepts :

	tokC		Number of tokens
	tokV		Token vector
	vX		vector index for the desired stat
	fnameP		Descriptive name for error reporting
	statusP		Ptr to current status variable

Returns :

	< value >	The stat obtained, (0 if there was a problem)
	*statusP	Set to FALSE if there was a problem,
			  otherwise left alone

*/

static int
statint( tokC, tokV, vX, fnameP, statusP )
	int		tokC;		/* Number of tokens */
	char		**tokV;		/* Token vector */
	int		vX;		/* Token selector */
	char		*fnameP;	/* Field name */
	int		*statusP;	/* Where to put false indicator */
{
DREG2	char		ch;		/* Char */
DREG1	int		v;		/* Value */
AREG1	char		*strP;		/* Ptr to string */
	char		msgbuf[80];

    /* Fail quietly if vX is not set; this should have already been
	noted when setting up the indexes. */
    if ( vX < 1 )
	return( 0 );

    /* Fail noisily if vX is past the token array.  */
    if ( vX >= tokC ) {
	rdb_error( "statint(): line does not contain field: ", fnameP );
	*statusP = FALSE;
	return( 0 );
    }

    /* Get the value */
    strP = tokV[vX];
    for( v = 0; ( ch = *strP++ ) != NUL; ) {
	if ( ( ch < '0' ) || ( ch > '9' ) ) {
	    sprintf( &msgbuf[0], "statint(): field \"%s\" (%d) has non-digit.",
				fnameP, vX );
	    rdb_error( &msgbuf[0], "" );
	    *statusP = FALSE;
	    return( 0 );
	}

	v = ( v * 10 ) + ( ch - '0' );
    }

    return( v );
}
/*

*//* statfixp( scale, tokC, tokV, vX, fnameP, statusP )

	Obtain an integer statistic from raw fixed-point data

Accepts :

	scale		Scale expressed by decimal point
	tokC		Number of tokens
	tokV		Token vector
	vX		vector index for the desired stat
	fnameP		Descriptive name for error reporting
	statusP		Ptr to current status variable

Returns :

	< value >	The stat obtained, (0 if there was a problem)
	*statusP	Set to FALSE if there was a problem,
			  otherwise left alone

*/

static int
statfixp( scale, tokC, tokV, vX, fnameP, statusP )
	int		scale;		/* The scale.. */
	int		tokC;		/* Number of tokens */
	char		**tokV;		/* Token vector */
	int		vX;		/* Token selector */
	char		*fnameP;	/* Field name */
	int		*statusP;	/* Where to put false indicator */
{
DREG1	char		ch;		/* Char */
AREG1	char		*strP;		/* Ptr to string */
AREG2	char		*destP;
	char		intpart[10], fracpart[10];
	char		msgbuf[80];

    /* Fail quietly if vX is not set; this should have already been
	noted when setting up the indexes. */
    if ( vX < 1 )
	return( 0 );

    /* Fail noisily if vX is past the token array.  */
    if ( vX >= tokC ) {
	rdb_error( "statfixp(): line does not contain field: ", fnameP );
	*statusP = FALSE;
	return( 0 );
    }

    /* Separate the value into integer and fractional parts */
    strP = tokV[vX];
    destP = &intpart[0];
    while( ( ch = *strP++ ) != NUL ) {
	if ( !isdigit( ch ) )
	    break;
	*destP++ = ch;
    }
    *destP = NUL;
    destP = &fracpart[0];
    if ( ch == '.' ) {
	while( ( ch = *strP++ ) != NUL ) {
	    if ( !isdigit( ch ) )
		break;
	    *destP++ = ch;
	}
    }
    *destP = NUL;

    if ( ch != NUL ) {
	sprintf( &msgbuf[0], "statfixp(): field \"%s\" (%d) has non-digit.",
				fnameP, vX );
	rdb_error( &msgbuf[0], "" );
	*statusP = FALSE;
	return( 0 );
    }

    return( ( atoi( &intpart[0] ) * scale ) + atoi( &fracpart[0] ) );
}
/*

*//******************************************************************
*                                                                   *
*               Support routines for stat AVL tree                  *
*                                                                   *
*********************************************************************/
/*

*//* pd_cmpkey( nameP, nodeP )

	Compare name keys for avl tree routines

Accepts :

	nameP		Ptr to target name struct
	nodeP		Ptr to node to test

Returns :

	< value >	-1, 0, or 1, as dictated by the rules of
			interface between the avl tree routines and
			the key comparison routine.

Note:

	This routine assumes that the first member of the PNODE structure
	is the AVL tree node structure, passed.  That is, that nodeP
	points to the top of the PNODE struct.

*/

static int
pd_cmpkey( nameP, nodeP )
	PDB_PNAME	*nameP;		/* Addr of new name */
	PNODE		*nodeP;		/* Addr of node */
{
	int		cmpval;		/* Comparison value */

    /* Check out last name */
    cmpval = strsicmp( &nameP->pn_lname[0], nodeP->pn_lnameP );
    if ( cmpval != 0 )
	return( cmpval );

    /* Last name passes... further check by matching first initial */
    if ( nameP->pn_fname[0] != NUL ) {
	cmpval = strsicmp( &nameP->pn_fname[0], nodeP->pn_fnameP );
	if ( cmpval != 0 )
	    return( cmpval );
    }

#ifdef	MATCHTEAM
    /* Still a match -- match on the team name */
    if ( ( nameP->pn_team[0] != NUL ) && ( nameP->pn_team[0] != ' ' )  &&
	 ( nodeP->pn_team[0] != NUL ) )
	cmpval = strsicmp( &nameP->pn_team[0], &nodeP->pn_team[0] );
#endif	/* MATCHTEAM */

	if ( nameP->batter == nodeP->batter )
		cmpval =  0;
	else
		cmpval = 1;

    return( cmpval );
}
/*

*//* pd_mknode( treeP, nameP, dataP, enodeP )

	Make a player node on behalf of AVL tree routines

Accepts :

	treeP		Address of tree header block
	nameP		Address of the player name struct for new node
	dataP		Address of data (not used)
	enodeP		Address of any existing node

Returns :

	<value>		Address of the new node


Notes :

	This routine assumes that the first member of the PNODE structure
	is the AVL tree node structure, passed.  That is, that nodeP
	points to the top of the PNODE struct.

	It also assumes that the AVLTREE struct is the first element of
	the player header struct (i.e., that treeP also points to the
	PDHDR struct).

*/

static AVLNODE *
pd_mknode( treeP, nameP, dataP, enodeP )
	AVLTREE		*treeP;		/* Addr of tree header block */
	PDB_PNAME	*nameP;		/* Addr of name struct */
	void		*dataP;		/* Addr of new data (n/a) */
	AVLNODE		*enodeP;	/* Existing node... */
{
	int		intC;		/* Number of intervals */
	int		nodesize;	/* Size of node required */
	PNODE		*nodeP;		/* New PNODE that we make */
	PDHDR		*hdrP;		/* Ptr to header struct */

    /* If enodeP is non-NULL, this is a call to overwrite the data.
       That should never happen, so return NULL in that case. */
    if ( enodeP != NULL )
	return( (AVLNODE *)NULL );


    /* Make a new node with space for all required intervals. */
    hdrP = (PDHDR *)treeP;
    intC = ( hdrP->pd_lastint - hdrP->pd_firstint ) +1;
    nodesize = sizeof( PNODE ) +
		intC * sizeof(INTSTAT) +
		strlen( &nameP->pn_lname[0] ) +1 +
		strlen( &nameP->pn_fname[0] ) +1;

    nodeP = (PNODE *)emalloc( "pd_mknode", "player stat node", nodesize );

    /* Setup the node struct */
    strcpy( &nodeP->pn_team[0], &nameP->pn_team[0] );
    nodeP->pn_istatP = (INTSTAT *)&nodeP[1];
    memset( nodeP->pn_istatP, 0, intC * sizeof(INTSTAT) );
    nodeP->pn_lnameP = (char *)&nodeP->pn_istatP[intC];
    strcpy( nodeP->pn_lnameP, &nameP->pn_lname[0] );
    nodeP->pn_fnameP = nodeP->pn_lnameP + strlen( nodeP->pn_lnameP ) +1;
    strcpy( nodeP->pn_fnameP, &nameP->pn_fname[0] );
    nodeP->pn_pos = 0;

	 nodeP->pdata.status[0] = '\0';
	 nodeP->pdata.date[0] = '\0';
	 nodeP->pdata.week = 0;
	 nodeP->pdata.zorch = PLAYER_FREE;

	 nodeP->batter = nameP->batter;

    /* Return the address of the AVL node member */
    return( &nodeP->pn_avl );
}
/*

*//* pd_rmnode( treeP, nodeP )

	Delete a node on behalf of AVL tree routines.

Accepts :

	treeP		Address of tree header block
	nodeP		Address of the node being removed

Returns :

	<value>			0 for success, -1 for failure

Notes :

	This routine assumes that the first member of the PNODE structure
	is the AVL tree node structure, passed.  That is, that nodeP
	points to the top of the PNODE struct.

*/

static int
pd_rmnode( treeP, nodeP )
	AVLTREE		*treeP;
	PNODE		*nodeP;
{
    /* Deallocate the node */
    free( (char *)nodeP );

    /* Return OK */
    return( 0 );
}
